#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QtQuick/QQuickWindow>
#include <QtQuick/QQuickItem>
#include <QSettings>
#include <QQmlContext>
#include <QDebug>

#include <QProcess>
#include <QQuickWindow>
#include <QScreen>
#include <QThread>

#include <QQmlPropertyMap>

#include "beeper.h"
#include "backlight.h"
#include "common.h"
#include "serialcontroller.h"
#include "gpiocontroller.h"
#include "gpiopin.h"
#include "mystyle.h"
#include "sound.h"
#include "system.h"


StyleValues MyStyle;
System mySystem;

int main(int argc, char *argv[])
{
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QGuiApplication app(argc, argv);
    QQmlApplicationEngine engine;

    serialcontroller serialController;
    GpioController gpioController;
    GpioPin fiber;
    GpioPin air_on;
    GpioPin gas_on;
    GpioPin laser;
    GpioPin interlock;
    GpioPin shuttleArm;
    GpioPin shuttleFiber;
    Beeper startup, beep, clicker,swoosh, beep2;
    Backlight backlight;

    qmlRegisterType < GpioController > ("GpioController.reachtech", 1, 0, "Gpio");
    qmlRegisterType < Beeper > ("sound.reachtech", 1, 0, "Beeper");
    qmlRegisterType < System > ("system.reachtech", 1, 0, "System");

    //Load the MyStyle variable into the engine where the QML will also be loaded
    engine.rootContext()->setContextProperty("MyStyle", &MyStyle);

    //set proper sound files for the beepers
    beep.setSoundFile("/data/share/audio/beep.wav");
    beep2.setSoundFile("/data/share/audio/beep2.wav");
    clicker.setSoundFile("/data/share/audio/click.wav");
    swoosh.setSoundFile("/data/share/audio/swoosh.wav");
    startup.setSoundFile("/data/share/audio/startup.wav");

    QScreen *screen = QGuiApplication::primaryScreen();
    QRect screenGeometry = screen->geometry();
    qDebug() << "Screen Size is " << screenGeometry.width() << "x" << screenGeometry.height();

    MyStyle.insert("screenWidth", screenGeometry.width());
    MyStyle.insert("screenHeight", screenGeometry.height());
    MyStyle.insert("screenFactor", screenGeometry.height());
    qDebug() << "Stored screen Size is"  << MyStyle.value("screenWidth").toInt() << "x" <<  MyStyle.value("screenHeight").toInt();


    qDebug()<<"Setting Pin Directions";

    gpioController.setDirection(0,"in"); //set pin 0 to input   'Fiber In
    gpioController.setDirection(1,"out"); //set pin 1 to output  'Air Compressor On/Off
    gpioController.setDirection(2,"out"); //set pin 2 to output  'Select External Tank (on) or Compressed Air (off)
    gpioController.setDirection(3,"out"); //set pin 3 to output 'Arm/Fiber
    gpioController.setDirection(4,"in"); //set pin 4 to input   'Air is On
    gpioController.setDirection(5,"in"); //set pin 5 to input   'Gas is On
    gpioController.setDirection(6,"in"); //set pin 6 to input   'Laser is Ready
    gpioController.setDirection(7,"in"); //set pin 7 to input   'Interlock OK
    gpioController.setDirection(8,"out");//set pin 8 to output 'Pilot Laser 1 On/Blink
    gpioController.setDirection(9,"out");//set pin 9 to output 'Pilot Laser 2 On/Blink
    gpioController.setDirection(10,"out");//set pin 10 to output 'Pilot Intense 1
    gpioController.setDirection(11,"out");//set pin 11 to output 'Pilot Intense 2
    gpioController.setDirection(12,"out");//set pin 12 to output 'Shuttle L/R
    gpioController.setDirection(13,"in");//set pin 13 to input 'Fiber Shuttle End Pos Sensor
    gpioController.setDirection(14,"in");//set pin 14 to input 'Arm Shuttle End Pos Sensor

    //Set the input pins to be monitored for changes
    fiber.setNumber(0);
    fiber.setPoll(1);
    air_on.setNumber(4);
    air_on.setPoll(true);
    gas_on.setNumber(5);
    gas_on.setPoll(true);
    laser.setNumber(6);
    laser.setPoll(true);
    interlock.setNumber(7);
    interlock.setPoll(true);
    shuttleFiber.setNumber(13);
    shuttleFiber.setPoll(true);
    shuttleArm.setNumber(14);
    shuttleArm.setPoll(true);

    startup.setVolume(100); //set Volume to 100%

    //Move the startup to its own thread so it runs all by itself in background.
    // No waiting for completion of the beep...
    QThread *thread = new QThread();
    startup.moveToThread(thread);
    thread->start();

    //Move the beeping, click, and swoosh sounds to their own thread so that it won't interfere with operations
    QThread *clickthread = new QThread();
    clicker.moveToThread(clickthread);
    clickthread->start();
    QThread *swooshthread = new QThread();
    swoosh.moveToThread(swooshthread);
    swooshthread->start();
    QThread *beepthread = new QThread();
    beep.moveToThread(beepthread);
    beepthread->start();
    QThread *beep2thread = new QThread();
    beep2.moveToThread(beepthread);
    beep2thread->start();

    //Load the QML into the engine
    engine.load(QUrl(QStringLiteral("qrc:/main.qml")));
    if (engine.rootObjects().isEmpty())
      return -1;

    //Load the engine into the device and open it
    QObject *toplevel = engine.rootObjects().value(0);
    QQuickWindow *window = qobject_cast<QQuickWindow *>(toplevel);
    if(window == nullptr){
        qDebug()<<"Can't instantiate window";
    }

    //Slot & Signal Connections
    //Allow the QML to send serial commands by connecting it to the serialController readyRead
    bool success = QObject::connect(window, SIGNAL(submitTextField(QString)), &serialController, SLOT(send(QString)));
    Q_ASSERT(success);
    //Connects the QML to the gpiocontroller to allow it to toggle IOs
    success = QObject::connect(window, SIGNAL(toggle(int)), &gpioController, SLOT(toggle(int)));
    Q_ASSERT(success);
    //Monitors all the various inputs and when they change the QML performs the linked function
    success = QObject::connect(&fiber, SIGNAL(pinValueChanged(QVariant)),window,SLOT(fiberIn(QVariant)));
    Q_ASSERT(success);
    success = QObject::connect(&air_on, SIGNAL(pinValueChanged(QVariant)),window,SLOT(airIn(QVariant)));
    Q_ASSERT(success);
    success = QObject::connect(&gas_on, SIGNAL(pinValueChanged(QVariant)),window,SLOT(gasIn(QVariant)));
    Q_ASSERT(success);
    success = QObject::connect(&laser, SIGNAL(pinValueChanged(QVariant)),window,SLOT(laserIn(QVariant)));
    Q_ASSERT(success);
    success = QObject::connect(&interlock, SIGNAL(pinValueChanged(QVariant)),window,SLOT(interlockIn(QVariant)));
    Q_ASSERT(success);
    success = QObject::connect(&shuttleArm, SIGNAL(pinValueChanged(QVariant)),window,SLOT(shuttleArm(QVariant)));
    Q_ASSERT(success);
    success = QObject::connect(&shuttleFiber, SIGNAL(pinValueChanged(QVariant)),window,SLOT(shuttleFiber(QVariant)));
    Q_ASSERT(success);
    //When called from the QML, plays the specified sound.
    success = QObject::connect(window, SIGNAL(click()), &clicker, SLOT(beep(void)));
    Q_ASSERT(success);
    success = QObject::connect(window, SIGNAL(swoosh()), &swoosh, SLOT(beep(void)));
    Q_ASSERT(success);
    success = QObject::connect(window, SIGNAL(beep()), &beep, SLOT(beep(void)));
    Q_ASSERT(success);
    success = QObject::connect(window, SIGNAL(beep2()), &beep2, SLOT(beep(void)));
    Q_ASSERT(success);
    //Connects the serial read of the CW/SP mode to the QML to reflect the correct laser mode
    success = QObject::connect(&serialController, SIGNAL(modeChange(QVariant,QVariant)), window, SLOT(modeChange(QVariant,QVariant)));
    Q_ASSERT(success);
    //Connects the power display to the serial response
    success = QObject::connect(&serialController, SIGNAL(powerUpdate(QVariant)), window, SLOT(powUpdate(QVariant)));
    Q_ASSERT(success);
    //connects the cycle time display to the serial response
    success = QObject::connect(&serialController, SIGNAL(repUpdate(QVariant)), window, SLOT(repeatUpdate(QVariant)));
    Q_ASSERT(success);
    //connects the pulse width display to the serial response
    success = QObject::connect(&serialController, SIGNAL(pulseUpdate(QVariant)), window, SLOT(pulseUpdate(QVariant)));
    Q_ASSERT(success);
    //Connects the two-stage start of the ready/standby modes to the QML file
    success = QObject::connect(&serialController, SIGNAL(readyStandby(QVariant)), window, SLOT(standbyMode(QVariant)));
    Q_ASSERT(success);
    //Parses the various error messages to the QML
    success = QObject::connect(&serialController, SIGNAL(error(QVariant)), window, SLOT(errorMode(QVariant)));
    Q_ASSERT(success);
    //When the serial connection sends "lasing" the QML reflects this change
    success = QObject::connect(&serialController, SIGNAL(lasing(QVariant)), window, SLOT(lasing(QVariant)));
    Q_ASSERT(success);
    //Connect the backlight settings to the QML
    success = QObject::connect(window, SIGNAL(backlight(int)), &backlight, SLOT(backlightSet(int)));
    Q_ASSERT(success);
    //connect Volume control settings to QML
    success = QObject::connect(window, SIGNAL(volume(int)), &startup, SLOT(setVolume(int)));
    Q_ASSERT(success);


    //Play the startup sound when the QML loads
    startup.beep();

    return app.exec();
}
