import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Controls.Material 2.12
import QtQuick.Dialogs 1.2
import QtQuick.Extras 1.4
import QtQuick.Layouts 1.12
import QtQuick.Controls 2.5
import QtQuick.Window 2.12
import QtGraphicalEffects 1.12
import QtQuick.Shapes 1.15

import QtMultimedia 5.12

import "."

ApplicationWindow {
    objectName: "mainWindow"
    id: mainWindow
    visible: true
    width: MyStyle.screenWidth
    height: MyStyle.screenHeight
    color: MyStyle.bgcolor

    title: qsTr("Laser Engineering Medical Laser")

    property var serviceCount: 0 //variable to track proper service menu
    property var currentBrightness: 7 //What the LCD brightness is
    property var currentVolume: 100 //What the volume level is in %
    property var currentPower: 1 //Current power of the system
    property var aimBrightFlag: 0 //determines if the input has to be toggled on or off (0 = off, 1 = on)
    property var aimIntensity: 10 //current level of aiming beam intensity

    property var counter: 0 //counter for which error message we're on

    signal submitTextField(string text) //Sending commands over serial.
    signal toggle(int val) //Signal for changing GPIO values
    signal setP(int pin, int val) //for setting pins on Aiming Intensity
    signal click() //click sound
    signal swoosh() //swoosh sound
    signal beep() //beep sound
    signal beep2() //2nd beep sound
    signal backlight(int intensity) //Signal to change brightness
    signal volume(int vol) //signal to change volume

    //Function to get date and time
    function getDate() {
        var date = new Date();
        //var weekdays = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
        //var months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
        //var weekday = weekdays[date.getDay()];
        var month = date.getMonth()+1; //months[date.getMonth()];
        var day = date.getDate();
        var time = new Date().toLocaleTimeString([], {
            hour: '2-digit'
        }, {
            minute: '2-digit'
        }).toString()
        var hour = time.slice(1, 2) === ":" ? time.slice(0, 1) : time.slice(0, 2)
        var minute = time.slice(1, 2) === ":" ? time.slice(2, 4) : time.slice(3, 5)
        var secs = time.slice(1, 2) === ":" ? time.slice(5, 7) : time.slice(6, 8)
        return month + "/" + day + " " + hour + ":" + minute + ":" + secs
    }

    //Timer triggers an error if shuttle takes too long to go from one end to the other
    Timer {
        id: shuttleTO
        interval: 3000; running: false; repeat:false
        onTriggered: {
            console.log("Shuttle Timeout Error Triggered")
            log.append(getDate()+"Shuttle Error")
            armFiberSelectMouseArea.enabled = true
            if(!(MyStyle.errorValue&2)) { MyStyle.errorValue = MyStyle.errorValue + 2 }
        }
    }
    //Timer triggers if pressure sensor takes too long to activate after turning on purge
    Timer {
        id: purgeTO
        interval: 750; running: false; repeat:false
        onTriggered: {
            if(MyStyle.airStatus) {
                log.append(getDate()+"Compressed Air Pressure Error")
                console.log("Compressed Air Error")
                if(!(MyStyle.errorValue&8)) { MyStyle.errorValue = MyStyle.errorValue + 8 }
            } else {
                log.append(getDate()+"Gas Pressure Error")
                console.log("Gas Pressure Error")
                if(!(MyStyle.errorValue&4)) { MyStyle.errorValue = MyStyle.errorValue + 4 }
            }
        }
    }//timer
    //Timer to blink the aiming beam
    Timer {
        id: blinking
        interval: 500; running: false; repeat: true
        onTriggered: {
            //Check if the aiming button is active (aiming beam supposed to be on)
            if(MyStyle.aimingButton) {
                //if it is, toggle the aiming beam
                if(MyStyle.armMode) {toggle(8)} else {toggle(9)}
                if(MyStyle.aimOn) {
                    MyStyle.aimOn = false
                } else { MyStyle.aimOn = true }
            }
        }
    }//timer
    //Timer for pressing and holding power up
    Timer {
        id: powUpRep
        interval: 250; running: false; repeat: true
        onTriggered: {
            click()
            if(currentPower < 30){
                submitTextField("YB")
            }
        }
    }//timer
    //Timer for pressing and holding power down
    Timer {
        id: powDownRep
        interval: 250; running: false; repeat: true
        onTriggered: {
            click()
            submitTextField("YC")
        }
    }//timer
    //This repeats the beeping sound as long as teh footpedal is depressed
    Timer {
        id: lasingSound
        interval: 100; running:false; repeat: true
        onTriggered: {
            beep2()
        }
    }
    //Error Message Display
    Timer {
        id: errorMessages
        interval: 1000; running: false; repeat: true
        triggeredOnStart: true
        onTriggered: {
            errorMessages.interval = 1000
            if((MyStyle.errorValue&1)&&!(counter&1)) {
                counter = counter + 1
                statusText.text = "Interlock Off: Turn key to activate"
            } else if((MyStyle.errorValue&2)&&!(counter&2)) {
                counter = counter + 2
                statusText.text = "Shuttle Error"
            } else if((MyStyle.errorValue&4)&&!(counter&4)) {
                counter = counter + 4
                statusText.text = "Gas Pressure Error"
            } else if((MyStyle.errorValue&8)&&!(counter&8)) {
                counter = counter + 8
                statusText.text = "Compressed Air Pressure Error"
            } else if((MyStyle.errorValue&16)&&!(counter&16)) {
                counter = counter + 16
                statusText.text = "Interlock Error";
            } else if((MyStyle.errorValue&32)&&!(counter&32)) {
                counter = counter + 32
                statusText.text = "Shutter Error";
            } else if((MyStyle.errorValue&64)&&!(counter&64)) {
                counter = counter + 64
                statusText.text = "Footpedal Error";
            } else if((MyStyle.errorValue&128)&&!(counter&128)) {
                counter = counter + 128
                statusText.text = "Attenuator Error";
            } else {
                counter = 0
                errorMessages.interval = 1
            }
        }
    }

    Timer {
        id: aimingBrightnessUp
        interval: 100; running: false; repeat: false
        triggeredOnStart: false
        onTriggered: {
            toggle(10)
            mainWindow.aimIntensity = mainWindow.aimIntensity + 1
            aimingBrightnessUpMouseArea.enabled = true
            console.log("Aiming Value: "+mainWindow.aimIntensity)
        }
    }
    Timer {
        id: aimBrightUpRep
        interval: 50; running: false; repeat: true
        triggeredOnStart: true
        onTriggered: {
           if(mainWindow.aimIntensity<63) {
               toggle(10)
               if (aimBrightFlag) {
                   mainWindow.aimIntensity = mainWindow.aimIntensity + 1
                   console.log("Aiming Value: "+mainWindow.aimIntensity)
               }
               if(aimBrightFlag) { aimBrightFlag=0 } else { aimBrightFlag=1 }
               console.log("Aim Flag: "+aimBrightFlag)
           } else {
               this.stop()
               aimAUp.color=MyStyle.borderColor
           }
       }
    }

    Timer {
        id: aimingBrightnessDown
        interval: 100; running: false; repeat: false
        triggeredOnStart: false
        onTriggered: {
            toggle(11)
            mainWindow.aimIntensity = mainWindow.aimIntensity - 1
            aimingBrightnessDownMouseArea.enabled = true
            console.log("Aiming Value: "+mainWindow.aimIntensity)
        }
    }
    Timer {
        id: aimBrightDownRep
        interval: 100; running: false; repeat: true
        triggeredOnStart: true
        onTriggered: {
            if(mainWindow.aimIntensity>0) {
                toggle(11)
                if (aimBrightFlag) {
                    mainWindow.aimIntensity = mainWindow.aimIntensity - 1
                    console.log("Aiming Value: "+mainWindow.aimIntensity)
                }
                if(aimBrightFlag) { aimBrightFlag=0 } else { aimBrightFlag=1 }
                console.log("Aim Flag: "+aimBrightFlag)
            } else {
                this.stop()
                aimADown.color=MyStyle.borderColor
            }
        }
    }

    //Laser Engineering Logo

    Image {
        id: leiLogo
        x: 30
        y: 90
        width: 475
        fillMode: Image.PreserveAspectFit
        source: "Images/leiLogo.png"
    }
    MouseArea {
        x: 1230;y: 0
        height: 50;width: 50
        onClicked: if(mainWindow.serviceCount===2) { serviceMenu.open();mainWindow.serviceCount=0;console.log(mainWindow.serviceCount) } else {mainWindow.serviceCount = 0;console.log(mainWindow.serviceCount)}
    }
    MouseArea {
        height: 50;width:50
        onClicked: if(mainWindow.serviceCount===0) {mainWindow.serviceCount = 1;console.log(mainWindow.serviceCount)} else {mainWindow.serviceCount=0;console.log(mainWindow.serviceCount)}
    }
    MouseArea {
        x: 1230;y: 750
        height: 50;width: 50
        onClicked: if(mainWindow.serviceCount===1){mainWindow.serviceCount = 2;console.log(mainWindow.serviceCount)} else {mainWindow.serviceCount = 0;console.log(mainWindow.serviceCount)}
    }
    Popup {
        id: serviceMenu
        x:600;y:20
        width: 480;height:MyStyle.screenHeight-40
        background: Rectangle { color: "#33373a" }
        closePolicy: Popup.NoAutoClose
        transformOrigin: Popup.Right
        enter: Transition {
                NumberAnimation { property: "scale"; from: 0.0; to: 1.0 }
            }
        exit: Transition {
                NumberAnimation { property: "scale"; from: 1.0; to: 0.0 }
            }
        Text {
            x: 90
            width: 300;height: 40
            text: "Service Menu"
            color: "white"
            font.pointSize: 18
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }

        Button {
            height:30;width: 30
            background: Rectangle {color: "gray"}
            onClicked: {
                click()
                serviceMenu.close()
            }
            Text {
                anchors.fill: parent
                text: "X"
                font.pointSize: 12
                color: "black"
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
            }
        }//button
        //Diagnostic Input indicators. Should be deleted or commented out in production
        Text {
            height: 20; width: 100
            x: 175; y: 50
            text: "Inputs"
            font.pixelSize: 24
            color: "white"
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
        Rectangle {
            x: 20;y:75
            height: 210; width: 410
            color: "transparent"
            border.color: "white"
            border.width: 2

            StatusIndicator {
                id: fiberStatus
                x: 10
                y: 10
                height: 40
                width: 40
                Text {
                    height: 40
                    color: "white"
                    text: "Fiber In"
                    font.pointSize: 11
                    anchors.left: parent.right
                    anchors.leftMargin: 3
                    verticalAlignment: Text.AlignVCenter
                }
            }
            StatusIndicator {
                 id: airStatus
                 x: 205
                 y: 10
                 height: 40
                 width: 40
                 Text {
                     height: 40
                     color: "white"
                     text: "Air On/Off"
                     font.pointSize: 11
                     anchors.left: parent.right
                     anchors.leftMargin: 3
                     verticalAlignment: Text.AlignVCenter
                 }
             }
            StatusIndicator {
                id: laserStatus
                x: 10
                y: 60
                height: 40
                width: 40
                Text {
                    height: 40
                    color: "white"
                    text: "Laser Ready"
                    font.pointSize: 11
                    anchors.left: parent.right
                    anchors.leftMargin: 3
                    verticalAlignment: Text.AlignVCenter
                }
            }
            StatusIndicator {
                id: gasStatus
                x: 205
                y: 60
                height: 40
                width: 40
                Text {
                    height: 40
                    color: "white"
                    text: "Gas On/Off"
                    font.pointSize: 11
                    anchors.left: parent.right
                    anchors.leftMargin: 3
                    verticalAlignment: Text.AlignVCenter
                }
            }
            StatusIndicator {
                id:interlockStatus
                x: 205
                y: 110
                height: 40
                width: 40
                Text {
                    height: 40
                    color: "white"
                    text: "Interlock OK"
                    font.pointSize: 12
                    anchors.left: parent.right
                    anchors.leftMargin: 3
                    verticalAlignment: Text.AlignVCenter
                }
            }
            StatusIndicator {
                id:shuttleFiberStatus
                x: 10
                y: 110
                height: 40
                width: 40
                Text {
                    height: 40
                    color: "white"
                    text: "Shuttle@Fiber"
                    font.pointSize: 11
                    anchors.left: parent.right
                    anchors.leftMargin: 3
                    verticalAlignment: Text.AlignVCenter
                }
            }
            StatusIndicator {
                id:shuttleArmStatus
                x: 10
                y: 160
                height: 40
                width: 40
                Text {
                    height: 40
                    color: "white"
                    text: "Shuttle@Arm"
                    font.pointSize: 11
                    anchors.left: parent.right
                    anchors.leftMargin: 3
                    verticalAlignment: Text.AlignVCenter
                }
            }
        }
//Status/Debug Log of events that happened
        Text {
            x:20;y:300
            text: "Log:"
            font.pointSize: 10
            color: "white"
            horizontalAlignment: Text.AlignLeft
        }
        ScrollView {
            x: 20;y:320
            height: 370; width: 410
            contentHeight: 200; contentWidth: 410
            TextArea {
                id: log
                color: "white"
                x: 0
                y: 0
                width: parent.width
                height: parent.height
            }

        } //log
    }
    //Settings button for the popup
    Image {
        id: settingsButton
        x: 10;y: 750
        source: "Images/settings.png"
        MouseArea {
            id: settingsMA
            anchors.fill: parent
            onClicked: {
                click()
                settingsMenu.open()
            }
            onPressed: parent.source = "Images/settingsPressed.png"
            onReleased: parent.source = "Images/settings.png"
        }
    }
    //settings popup screen
    Popup {
        id: settingsMenu
        x: Math.round((parent.width - width) / 2)
        y: Math.round((parent.height - height) / 2)
        width: 275
        height: 315
        modal: true
        dim: true
        focus: true

        closePolicy: Popup.NoAutoClose
        background: Rectangle {
                color: "#33373a"
                border.color: "black"
        }
        transformOrigin: Popup.BottomLeft
        enter: Transition {
                NumberAnimation { property: "opacity"; from: 0.0; to: 1.0 }
            }
        exit: Transition {
                NumberAnimation { property: "opacity"; from: 1.0; to: 0.0 }
            }
        Button {
            x: parent.width-width
            y: 0
            width: 30
            height: 30
            onClicked: {
                click()
                settingsMenu.close()
            }
            Text {
                anchors.fill: parent
                text: "X"
                font.pointSize: 12
                color: "black"
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
            }
        }
        Text {
            height: 40
            width: parent.width
            text: "Settings"
            color: "white"
            font.pointSize: 14
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
//pilotMode button is for toggling whether the aiming beam will be firing while the process beam is also firing.
//When pilotMode is activated, the aiming beam will stay on while lasing.
//if it is deactivated, the aiming beam will shut off while lasing, but will turn back on afterwards.
        Rectangle {
            id: pilotMode
            x: 27
            y: 60
            width: 200
            height: 50
            color: if(MyStyle.pilotWhileOn) { MyStyle.borderColor } else { MyStyle.bgcolor }
            border.color: MyStyle.borderColor
            border.width: 2
            radius: 25
            Text {
                color: if(MyStyle.pilotWhileOn) { MyStyle.lightTextColor } else { MyStyle.darkTextColor }
                text: if(MyStyle.pilotWhileOn) { qsTr("On w/ Footswitch") } else { qsTr("Off w/ Footswitch") }
                font.pointSize: 12
                anchors.fill: parent
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
            }
            MouseArea {
                id: pilotModeMouseArea
                anchors.fill: parent
                onClicked: click()
                onPressed: parent.color = MyStyle.pressedColor
                onReleased: {
                    if(MyStyle.pilotWhileOn) { MyStyle.pilotWhileOn = false } else { MyStyle.pilotWhileOn = true }
                    if(MyStyle.pilotWhileOn) {
                                parent.color = MyStyle.borderColor
                            } else {
                                parent.color = MyStyle.bgcolor
                            }
                }
            }
        }
//Setting the brightness of the LCD
        Text {
            x: 39; y: 120
            width: 176
            horizontalAlignment: Text.AlignHCenter
            text: "Brightness"
            color: "white"
        }
        Shape {
            x: 14; y: 145
            height: 50; width: 25
            ShapePath {
                id: blightDown
                strokeWidth: 1
                fillColor: "red"
                startX: 25; startY: 0
                PathLine { x: 25; y: 50 }
                PathLine { x: 0; y: 25 }
                PathLine { x: 25; y: 0 }
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    click()
                    if(mainWindow.currentBrightness>1) {
                        mainWindow.currentBrightness = mainWindow.currentBrightness - 1
                        backlight(mainWindow.currentBrightness)
                        log.append(getDate()+": Brightness set to "+(100/7*mainWindow.currentBrightness).toFixed(0)+"%")
                    }
                }
            }
        }
        Rectangle {
            x: 39; y: 145
            height: 50; width: 176
            color: "transparent"
            border.width: 1
            border.color: "white"
            Rectangle {
                x: 0
                y: 2
                id: bl1
                width: 25
                height: 46
                color: "green"
            }
            Rectangle {
                x: 25
                y: 2
                id: bl2
                width: 25
                height: 46
                color: if(mainWindow.currentBrightness>1){"green"}else{"transparent"}
            }
            Rectangle {
                x: 50
                y: 2
                id: bl3
                width: 25
                height: 46
                color: if(mainWindow.currentBrightness>2){"green"}else{"transparent"}
            }
            Rectangle {
                x: 75
                y: 2
                id: bl4
                width: 25
                height: 46
                color: if(mainWindow.currentBrightness>3){"green"}else{"transparent"}
            }
            Rectangle {
                x: 100
                y: 2
                id: bl5
                width: 25
                height: 46
                color: if(mainWindow.currentBrightness>4){"green"}else{"transparent"}
            }
            Rectangle {
                x: 125
                y: 2
                id: bl6
                width: 25
                height: 46
                color: if(mainWindow.currentBrightness>5){"green"}else{"transparent"}
            }
            Rectangle {
                x: 150
                y: 2
                id: bl7
                width: 25
                height: 46
                color: if(mainWindow.currentBrightness>6){"green"}else{"transparent"}
            }
        }
        Shape {
            x: 215; y: 145
            height: 50; width: 25
            ShapePath {
                id: blightUp
                strokeWidth: 1
                fillColor: "red"
                startX: 0; startY: 0
                PathLine { x: 0; y: 50 }
                PathLine { x: 25; y: 25 }
                PathLine { x: 0; y: 0 }
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    click()
                    if(mainWindow.currentBrightness<7) {
                        mainWindow.currentBrightness = mainWindow.currentBrightness + 1
                        backlight(mainWindow.currentBrightness)
                        console.log("Current Brightness at "+mainWindow.currentBrightness)
                        log.append(getDate()+": Brightness set to "+(100/7*mainWindow.currentBrightness).toFixed(0)+"%")
                    }
                }
            }
        }

//Volume Control
        Text {
            x: 82; y: 200
            width: 90
            horizontalAlignment: Text.AlignHCenter
            text: "Volume"
            color: "white"
        }
        Shape {
            x: 57; y: 225
            height: 50; width: 25
            ShapePath {
                id: volDown
                strokeWidth: 1
                fillColor: "red"
                startX: 25; startY: 0
                PathLine { x: 25; y: 50 }
                PathLine { x: 0; y: 25 }
                PathLine { x: 25; y: 0 }
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if(mainWindow.currentVolume>0) {
                        mainWindow.currentVolume = mainWindow.currentVolume - 5
                        volume(mainWindow.currentVolume)
                        console.log("Current Volume at "+mainWindow.currentVolume)
                        log.append(getDate()+"Volume set to "+mainWindow.currentVolume+"%")
                    }
                    beep()
                }
            }
        }
        Rectangle {
            x: 82; y: 225
            height: 50; width: 90
            color: "transparent"
            border.width: 2
            border.color: "white"
            Text {
                text: (mainWindow.currentVolume+"%")
                color: "white"
                anchors.fill: parent
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                font.pointSize: 18
            }
        }

        Shape {
            x: 173; y: 225
            height: 50; width: 25
            ShapePath {
                id: volUp
                strokeWidth: 1
                fillColor: "red"
                startX: 0; startY: 0
                PathLine { x: 0; y: 50 }
                PathLine { x: 25; y: 25 }
                PathLine { x: 0; y: 0 }
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if(mainWindow.currentVolume<100) {
                        mainWindow.currentVolume = mainWindow.currentVolume + 5
                        volume(mainWindow.currentVolume)
                        console.log("Current Volume at "+mainWindow.currentVolume)
                        log.append(getDate()+"Volume set to "+mainWindow.currentVolume+"%")
                    }
                    beep()
                }
            }
        }
    }
//Settings Menu End

    //Ext. Tank Gas Button
    //Toggles GPIO (J22) pin 1. Default at system start is off
    //MyStyle.gasStatus is used to determine if the gas is currently on or not. This is set separately from the GPIO input as a means of double validation
    Rectangle {
        id: gas
        x: 240
        y: 250
        width: 200
        height: 50
        color: if(MyStyle.gasStatus){MyStyle.borderColor} else {MyStyle.bgcolor}
        border.color: MyStyle.borderColor
        border.width: 2
        radius: 25
        Text {
            id: gasText
            color: if(MyStyle.gasStatus){ MyStyle.lightTextColor } else { MyStyle.darkTextColor }
            text: qsTr("Gas Off")
            font.pointSize: 14
            anchors.fill: parent
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
        }
        MouseArea {
            id: gasMouseArea
            anchors.fill: parent
            onClicked: {
                click()
                //if compressed air is on, turn it off first
                if(MyStyle.airStatus) {
                    toggle(2)
                    MyStyle.airStatus = false
                }
                //turn on external tank
                toggle(1)
                if(MyStyle.gasStatus) {
                    MyStyle.gasStatus = false
                    //If the there is a gas pressure Error, clear it
                    statusBar.color = armFiberModeLight.border.color; standbyAreaMouseArea.enabled = true
                    if(MyStyle.errorValue&4) {MyStyle.errorValue = MyStyle.errorValue - 4 }
                } else { MyStyle.gasStatus = true; purgeTO.start() }


            }
            onPressed: parent.color = MyStyle.pressedColor
        }
    }
    //Compressed Air Button
    //Toggles GPIO (J22) pin 2. Default at system start is off
    //MyStyle.airStatus keeps track of if the button (and input) is on or off. This is done separately from the GPIO input as a means of double validation.
    Rectangle {
        id: air
        x: 30
        y: 250
        width: 200
        height: 50
        color: if(MyStyle.airStatus) { MyStyle.borderColor } else { MyStyle.bgcolor }
        border.color: MyStyle.borderColor
        border.width: 2
        radius: 25
        Text {
            id: airText
            color: if(MyStyle.airStatus){ MyStyle.lightTextColor } else { MyStyle.darkTextColor }
            text: qsTr("Comp. Air Off")
            font.pointSize: 14
            anchors.fill: parent
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
        }
        MouseArea {
            id: airMouseArea
            anchors.fill: parent
            onClicked: {
                click()
                //if external gas tank is on, turn it off first
                if(MyStyle.gasStatus) {
                    toggle(1)
                    MyStyle.gasStatus = false
                }
                toggle(2)
                if(MyStyle.airStatus) {
                    MyStyle.airStatus = false
                    //If the there is a gas pressure Error, clear it
                    statusBar.color = armFiberModeLight.border.color; standbyAreaMouseArea.enabled = true
                    if(MyStyle.errorValue&8) {MyStyle.errorValue = MyStyle.errorValue - 8 }
                } else { MyStyle.airStatus = true; purgeTO.start() }
            }
            onPressed: parent.color = MyStyle.pressedColor
        }
    }

    //Laser Power adjustment buttons
    //Increases the laser power by 1 step
    Rectangle {
        id: powUp
        width: powerDisplayBorder.width
        height: this.width
        color: MyStyle.borderColor
        radius: this.width/2
        border.color: MyStyle.borderColor
        border.width: 3
        anchors.left: powerDisplayBorder.left
        anchors.top: powerDisplayBorder.top
        anchors.topMargin: -this.width/2
        Text {
            height: this.width/2
            color: MyStyle.lightTextColor
            text: qsTr("+")
            anchors.right: parent.right
            anchors.left: parent.left
            anchors.top: parent.top
            font.pointSize: parent.width/4
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
        }
        MouseArea{
            id: powerUpButtonMouseArea
            height: parent.height/2
            anchors.right: parent.right
            anchors.left: parent.left
            anchors.top: parent.top
            onClicked: {
                if((statusText.text == "Power Low Warning")|(statusText.text == "Power High Warning")) { statusText.text = "" }
                click()
                if(currentPower < 30){
                    submitTextField("YB")
                }
            }
            onPressAndHold: {
                powUpRep.start()
                if((statusText.text == "Power Low Warning")|(statusText.text == "Power High Warning")) { statusText.text = "" }
            }
            onPressed: {parent.color=MyStyle.pressedColor}
            onReleased: {
                parent.color=MyStyle.borderColor
                powUpRep.stop()
            }
        }
    }
    //Decrease the laser power by 1 step
    Rectangle {
        id: powDown
        width: powerDisplayBorder.width
        height: this.width
        color: MyStyle.borderColor
        radius: this.width/2
        anchors.left: powerDisplayBorder.left
        anchors.right: powerDisplayBorder.right
        anchors.bottom: powerDisplayBorder.bottom
        anchors.bottomMargin: -this.width/2
        border.color: MyStyle.borderColor
        border.width: 3
        Text {
            height: this.width/2
            color: MyStyle.lightTextColor
            text: qsTr("-")
            anchors.right: parent.right
            anchors.left: parent.left
            font.pointSize: parent.width/4
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            anchors.bottom: parent.bottom
        }
        MouseArea{
            id: powerDownButtonMouseArea
            width: parent.width
            height: this.width/2
            visible: true
            anchors.right: parent.right
            anchors.left: parent.left
            anchors.bottom: parent.bottom
            onClicked: {
               if((statusText.text == "Power Low Warning")|(statusText.text == "Power High Warning")) { statusText.text = "" }
                click()
                submitTextField("YC")
            }
            onPressAndHold: {
                powDownRep.start()
                if((statusText.text == "Power Low Warning")|(statusText.text == "Power High Warning")) { statusText.text = "" }
            }
            onPressed: {parent.color=MyStyle.pressedColor}
            onReleased: {
                parent.color=MyStyle.borderColor
                powDownRep.stop()
            }
        }
    }
    //Power display
    Rectangle {
        id: powerDisplayBorder
        x: (1280/2) - (this.width/2)
        y: ((800-statusBar.height)/2) + statusBar.height - (this.height/2)
        width: 300
        height: this.width
        color: MyStyle.bgcolor
        border.width: 3
        border.color: MyStyle.borderColor

        Text {
            id: powerDisplay
            height: 240
            text: qsTr("--")
            anchors.right: parent.right
            anchors.left: parent.left
            anchors.top: parent.top
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: parent.width*0.6
            color: MyStyle.darkTextColor
        }
        Text {
            id: powerUnits
            height: 60
            text: qsTr("watts")
            anchors.bottom: parent.bottom
            anchors.right: parent.right
            anchors.left: parent.left
            verticalAlignment: Text.AlignTop
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 50
            color: MyStyle.darkTextColor
        }
    }
    //These objects control the laser mode: CW or SP
    //All objects except the SP are drawn on and colors from the first line propogate through subsequent lines in a "shape"
    Rectangle {
        id: modeBox
        x: 90
        y: 550
        width: 300
        height: 100
        color: "#00000000"
        border.color: MyStyle.borderColor
        border.width: 3
        MouseArea{
            id: modeButtonMouseArea
            x: 0
            y: 0
            anchors.fill: parent
            onClicked: {
                click()
                submitTextField("YH")
            }
        }
        Rectangle {
            id: cwBox
            width: parent.width
            height: 50
            color: "#00000000"
            border.color: parent.border.color
            border.width: parent.border.width
            anchors.left: parent.left
            anchors.top: parent.top
            Text {
                x: 5
                height: 47
                text: qsTr("CW")
                verticalAlignment: Text.AlignVCenter
                font.pixelSize: 24
                color: cwGraphic.color
            }
            Rectangle {
                id: cwGraphic
                x: 50
                width: 240
                height: 2
                border.width: 0
                anchors.verticalCenter: parent.verticalCenter
                color: MyStyle.darkTextColor
            }
        }
        Rectangle {
            id: spBox
            width: parent.width
            height: 53
            color: "#00000000"
            border.color: parent.border.color
            border.width: parent.border.width
            anchors.left: parent.left
            anchors.top: cwBox.bottom
            anchors.topMargin: -3
            Text {
                x: 5
                height: 53
                color: pulsedGraphic.strokeColor
                text: qsTr("SP")
                verticalAlignment: Text.AlignVCenter
                font.pixelSize: 24
            }
            Shape {
                x: 50
                y: 16
                height: 42
                width: 40
                ShapePath {
                    id: pulsedGraphic
                    strokeWidth: 2
                    strokeColor: "black"
                    fillColor: "transparent"
                    startX: 0; startY: 20
                    PathLine { x: 5; y: 0}
                    PathLine { x: 10; y: 20}
                    PathLine { x: 15; y: 0}
                    PathLine { x: 20; y: 20}
                    PathLine { x: 25; y: 0}
                    PathLine { x: 30; y: 20}
                    PathLine { x: 35; y: 0}
                    PathLine { x: 40; y: 20}
                    PathLine { x: 45; y: 0}
                    PathLine { x: 50; y: 20}
                    PathLine { x: 55; y: 0}
                    PathLine { x: 60; y: 20}
                    PathLine { x: 65; y: 0}
                    PathLine { x: 70; y: 20}
                    PathLine { x: 75; y: 0}
                    PathLine { x: 80; y: 20}
                    PathLine { x: 85; y: 0}
                    PathLine { x: 90; y: 20}
                    PathLine { x: 95; y: 0}
                    PathLine { x: 100; y: 20}
                    PathLine { x: 105; y: 0}
                    PathLine { x: 110; y: 20}
                    PathLine { x: 115; y: 0}
                    PathLine { x: 120; y: 20}
                    PathLine { x: 125; y: 0}
                    PathLine { x: 130; y: 20}
                    PathLine { x: 135; y: 0}
                    PathLine { x: 140; y: 20}
                    PathLine { x: 145; y: 0}
                    PathLine { x: 150; y: 20}
                    PathLine { x: 155; y: 0}
                    PathLine { x: 160; y: 20}
                    PathLine { x: 165; y: 0}
                    PathLine { x: 170; y: 20}
                    PathLine { x: 175; y: 0}
                    PathLine { x: 180; y: 20}
                    PathLine { x: 185; y: 0}
                    PathLine { x: 190; y: 20}
                    PathLine { x: 195; y: 0}
                    PathLine { x: 200; y: 20}
                    PathLine { x: 205; y: 0}
                    PathLine { x: 210; y: 20}
                    PathLine { x: 215; y: 0}
                    PathLine { x: 220; y: 20}
                    PathLine { x: 225; y: 0}
                    PathLine { x: 230; y: 20}
                    PathLine { x: 235; y: 0}
                    PathLine { x: 240; y: 20}
                }
            }
        }
    }
    Rectangle {
        id: cwPulseBox
        width: 103
        height: 103
        color: "#00000000"
        anchors.left: modeBox.left
        anchors.top: modeBox.bottom
        anchors.topMargin: -3
        border.color: modeBox.border.color
        border.width: modeBox.border.width
        Rectangle {
            id: cwPulseGraphic1
            x: 10
            y: 10 + cwPulseGraphic2.height
            height: 2
            width: 10
            color: MyStyle.darkTextColor
            border.width: 0

        }
        Rectangle {
            id: cwPulseGraphic2
            x: cwPulseGraphic1.x + cwPulseGraphic1.width
            y: 12
            height: 80 + cwPulseGraphic1.height
            width: 2
            color: cwPulseGraphic1.color
            border.width: 0

        }
        Rectangle {
            id: cwPulseGraphic3
            x: cwPulseGraphic1.x + cwPulseGraphic1.width
            y: cwPulseGraphic2.y
            height: 2
            width: 70
            color: cwPulseGraphic1.color
            border.width: 0
        }
    }
    Rectangle {
        id: spPulseBox
        width: 100
        height: 103
        color: "#00000000"
        anchors.leftMargin: -3
        anchors.left: cwPulseBox.right
        anchors.top: modeBox.bottom
        anchors.topMargin: -3
        border.color: modeBox.border.color
        border.width: modeBox.border.width
        Rectangle {
            id: spPulseGraphic1
            x: 10
            y: 10 + spPulseGraphic2.height
            height: 2
            width: 10
            color: MyStyle.darkTextColor
            border.width: 0
        }
        Rectangle {
            id: spPulseGraphic2
            x: spPulseGraphic1.x + spPulseGraphic1.width
            y: 12
            height: 80 + spPulseGraphic1.height
            width: 2
            color: spPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: spPulseGraphic3
            x: spPulseGraphic1.x + spPulseGraphic1.width
            y: spPulseGraphic2.y
            height: 2
            width: 15
            color: spPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: spPulseGraphic4
            x: spPulseGraphic3.x + spPulseGraphic3.width
            y: 12
            height: spPulseGraphic2.height
            width: 2
            color: spPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: spPulseGraphic5
            x: spPulseGraphic3.x + spPulseGraphic3.width
            y: 10 + spPulseGraphic2.height
            height: 2
            width: 55
            color: spPulseGraphic1.color
            border.width: 0
        }
    }
    Rectangle {
        id: sprPulseBox
        width: 103
        height: 103
        color: "#00000000"
        anchors.right: modeBox.right
        anchors.top: modeBox.bottom
        anchors.topMargin: -3
        border.color: modeBox.border.color
        border.width: modeBox.border.width
        Rectangle {
            id: sprPulseGraphic1
            x: 10
            y: 10 + sprPulseGraphic2.height
            height: 2
            width: 10
            color: MyStyle.darkTextColor
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic2
            x: sprPulseGraphic1.x + sprPulseGraphic1.width
            y: 12
            height: 80 + sprPulseGraphic1.height
            width: 2
            color: sprPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic3
            x: sprPulseGraphic1.x + sprPulseGraphic1.width
            y: sprPulseGraphic2.y
            height: 2
            width: 15
            color: sprPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic4
            x: sprPulseGraphic3.x + sprPulseGraphic3.width
            y: sprPulseGraphic2.y
            height: sprPulseGraphic2.height
            width: 2
            color: sprPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic5
            x: sprPulseGraphic3.x + sprPulseGraphic3.width
            y: 10 + sprPulseGraphic2.height
            height: 2
            width: 10
            color: sprPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic6
            x: sprPulseGraphic5.x + sprPulseGraphic5.width
            y: sprPulseGraphic2.y
            height: sprPulseGraphic2.height
            width: 2
            color: sprPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic7
            x: sprPulseGraphic5.x + sprPulseGraphic5.width
            y: sprPulseGraphic2.y
            height: 2
            width: 15
            color: sprPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic8
            x: sprPulseGraphic7.x + sprPulseGraphic7.width
            y: sprPulseGraphic2.y
            height: sprPulseGraphic2.height
            width: 2
            color: sprPulseGraphic1.color
            border.width: 0
        }
        Rectangle {
            id: sprPulseGraphic9
            x: sprPulseGraphic7.x + sprPulseGraphic7.width
            y: 10 + sprPulseGraphic2.height
            height: 2
            width: 35
            color: sprPulseGraphic1.color
            border.width: 0
        }
    }
    //This box holds the controls for changing the pulse width and the cycle time (if you aren't doing a single shot)
    Rectangle {
        id: pulseParameterBox
        x: 115
        y: 325
        width: 250
        height: 200
        color: "#00000000"
        border.width: 3
        border.color: MyStyle.borderColor
        //Buttons and display for the pulse width parameters
        Rectangle {
            id: pulseDisplayBorder
            x: 70
            y: 35
            width: 110
            height: 60
            color: MyStyle.bgcolor
            z: 1
            border.width: 3
            border.color: MyStyle.borderColor

            Text {
                id: pulseDisplay
                x: 0
                y: 0
                width: pulseDisplayBorder.width
                height: 60
                color: MyStyle.darkTextColor
                text: qsTr("0.05")
                z: 2
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                font.pixelSize: 35

                Text {
                    id: pulseUnit
                    x: 0
                    y: 0
                    width: 30
                    height: 60
                    color: MyStyle.darkTextColor
                    text: {
                        if (parent.text == "CONT") {
                            " "
                        } else {
                            "s"
                        }
                    }
                    z: 2
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: Text.AlignRight
                    anchors.right: parent.right
                    anchors.rightMargin: {
                        if (parent.text == "0.05") {
                            1
                        } else {
                            10
                        }
                    }
                    font.pixelSize: parent.font.pixelSize
                }
            }

            Text {
                id: pulseDisplayLabel
                width: parent.width
                height: 30
                color: MyStyle.darkTextColor
                text: qsTr("Pulse Width")
                anchors.bottom: parent.top
                verticalAlignment: Text.AlignBottom
                horizontalAlignment: Text.AlignHCenter
                anchors.bottomMargin: 5
                font.pixelSize: 24
            }
        }
        Rectangle {
            id: pulseUpBody
            x: 960
            y: pulseDisplayBorder.y
            width: 30
            height: 60
            color: MyStyle.borderColor
            border.color: this.color
            border.width: 0
            anchors.left: pulseDisplayBorder.right
            Text {
                width: 30
                height: 60
                anchors.left: parent.left
                anchors.leftMargin: 10
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 2
                color: MyStyle.lightTextColor
                text: "+"
                z: 2
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                font.pixelSize: 50
            }
            MouseArea{
                id: pulseUpButtonMouseArea
                anchors.right: pulseUpRound.right
                anchors.rightMargin: 0
                anchors.bottom: parent.bottom
                anchors.left: parent.left
                anchors.top: parent.top
                onClicked: {
                    click()
                    submitTextField("YD")
                }
                onPressed: {parent.color = MyStyle.pressedColor}
                onReleased: {parent.color = MyStyle.borderColor}
            }
            Rectangle {
                id: pulseUpRound
                width: 60
                height: 60
                color: parent.color
                radius: 30
                border.color: parent.color
                border.width: 3
                anchors.left: parent.right
                anchors.leftMargin: -30
            }
        }
        Rectangle {
            id: pulseDownBody
            x: 755
            y: pulseDisplayBorder.y
            width: 30
            height: 60
            color: MyStyle.borderColor
            anchors.right: pulseDisplayBorder.left
            anchors.rightMargin: 0
            border.color: this.color
            border.width: 0
            Text {
                width: 30
                height: 60
                anchors.left: parent.left
                anchors.leftMargin: -10
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 2
                color: MyStyle.lightTextColor
                text: "-"
                z: 2
                enabled: false
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                font.pixelSize: 50
            }
            MouseArea{
                id: pulseDownButtonMouseArea
                anchors.right: parent.right
                anchors.bottom: parent.bottom
                anchors.left: pulseDownRound.left
                anchors.top: parent.top
                onClicked: {
                    click()
                    submitTextField("YE")
                }
                onPressed: {parent.color=MyStyle.pressedColor}
                onReleased: {parent.color=MyStyle.borderColor}
            }
            Rectangle {
                id: pulseDownRound
                width: 60
                height: 60
                color: parent.color
                radius: 30
                border.color: parent.color
                border.width: 3
                anchors.right: parent.left
                anchors.rightMargin: -30
            }
        }

        //Buttons and display for the Cycle Time parameters
        Rectangle {
            id: repeatDisplayBorder
            x: 70
            y: 130
            width: 110
            height: 60
            color: MyStyle.bgcolor
            z: 1
            border.width: 3
            border.color: MyStyle.borderColor

            Text {
                id: repeatDisplay
                x: 0
                y: 0
                width: repeatDisplayBorder.width
                height: 60
                color: MyStyle.darkTextColor
                text: qsTr("OFF")
                z: 2
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                font.pixelSize: 35

                Text {
                    x: 0
                    y: 0
                    width: 30
                    height: 60
                    color: MyStyle.darkTextColor
                    text: {
                        if (parent.text == "OFF") {
                            " "
                        } else {
                            "s"
                        }
                    }
                    z: 2
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: Text.AlignRight
                    anchors.right: parent.right
                    anchors.rightMargin: 10
                    font.pixelSize: parent.font.pixelSize
                }
            }

            Text {
                id: repeatDisplayLabel
                width: parent.width
                height: 30
                color: MyStyle.darkTextColor
                text: qsTr("Cycle Time")
                anchors.bottom: parent.top
                verticalAlignment: Text.AlignBottom
                horizontalAlignment: Text.AlignHCenter
                anchors.bottomMargin: 5
                font.pixelSize: 24
            }
        }
        Rectangle {
            id: repeatUpBody
            x: 960
            y: repeatDisplayBorder.y
            width: 30
            height: 60
            color: MyStyle.borderColor
            border.color: this.color
            border.width: 0
            anchors.left: repeatDisplayBorder.right
            Text {
                width: 30
                height: 60
                anchors.left: parent.left
                anchors.leftMargin: 10
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 2
                color: MyStyle.lightTextColor
                text: "+"
                z: 1
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                font.pixelSize: 50
            }
            MouseArea{
                id: repeatUpButtonMouseArea
                anchors.right: repeatUpRound.right
                anchors.bottom: parent.bottom
                anchors.left: parent.left
                anchors.top: parent.top
                onClicked: {
                    click()
                    submitTextField("YF")
                }
                onPressed: {parent.color=MyStyle.pressedColor}
                onReleased: {parent.color=MyStyle.borderColor}
            }
            Rectangle {
                id: repeatUpRound
                width: 60
                height: 60
                color: parent.color
                radius: 30
                border.color: parent.color
                border.width: 3
                anchors.left: parent.right
                anchors.leftMargin: -30
            }
        }
        Rectangle {
            id: repeatDownBody
            x: 755
            y: repeatDisplayBorder.y
            width: 30
            height: 60
            color: MyStyle.borderColor
            anchors.right: repeatDisplayBorder.left
            border.color: this.color
            border.width: 0
            Text {
                width: 30
                height: 60
                z: 1
                anchors.left: parent.left
                anchors.leftMargin: -10
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 2
                color: MyStyle.lightTextColor
                text: "-"
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                font.pixelSize: 50
            }
            MouseArea{
                id: repeatDownButtonMouseArea
                anchors.right: parent.right
                anchors.bottom: parent.bottom
                anchors.left: repeatDownRound.left
                anchors.top: parent.top
                onClicked: {
                    click()
                    submitTextField("YG")
                }
                onPressed: {parent.color=MyStyle.pressedColor}
                onReleased: {parent.color=MyStyle.borderColor}
            }
            Rectangle {
                id: repeatDownRound
                width: 60
                height: 60
                color: parent.color
                radius: 30
                border.color: parent.color
                border.width: 3
                anchors.right: parent.left
                anchors.rightMargin: -30
            }
        }
    }
    //The status bar conveys information via color and text. These are applied when various state changes occur as described in the "Connections" portion of the code
    Rectangle {
        id: statusBar
        width: 750
        height: 70
        color: armFiberModeLight.border.color
        anchors.right: parent.right
        anchors.rightMargin: 0
        anchors.left: parent.left
        anchors.leftMargin: 0
        anchors.top: parent.top
        anchors.topMargin: 0
        z: 10
        visible: true

        Text {
            id: statusText
            text: ""
            font.bold: true
            verticalAlignment: Text.AlignBottom
            horizontalAlignment: Text.AlignHCenter
            anchors.fill: parent
            font.pixelSize: 60
        }
    }
    Rectangle {
        id: interlockBar
        height: 3
        color: if(MyStyle.interlock) { "#04f408" } else { "red" }
        anchors.right: parent.right
        anchors.left: parent.left
        anchors.top: statusBar.bottom
        visible: true
    }

    //The circular "light" on the right of the standby/ready indicator
    Rectangle {
        id: readyLight
        y: standbyArea.y
        z: 3
        width: 100
        height: 100
        color: "#ffee00"
        radius: 50
        anchors.left: standbyArea.right
        anchors.leftMargin: -50
        border.width: 7
        border.color: "#7a7474"

        ColorAnimation on color {
            id: rdyLtRdy
            from: "#ffee00"; to: "#00a854"
            duration: 500
            running: false
        }
        ColorAnimation on color {
            id: rdyLtStndby
            from: "#00a854";to: "#ffee00"
            duration: 500
            running: false
        }
    }
    //The main body of the standby/ready display area
    Rectangle {
        id: standbyArea
        x: 950
        y: 88
        z: 1
        width: 250
        height: 100
        color: MyStyle.standbyColor
        layer.smooth: true
        state: "standby"
        MouseArea{
            id: standbyAreaMouseArea
            x:0
            y:0
            width: 300
            height: 100
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.bottom: parent.bottom

            onClicked: {
                click()
                if(standbyText.text == "STANDBY"){
                    submitTextField("YA")
                } else {
                    submitTextField("YU")
                }
            }
        }
        ColorAnimation on color{
            id: readyAnim
            from: MyStyle.standbyColor; to: MyStyle.readyColor
            duration: 500
            running: false
        }
        ColorAnimation on color{
            id: standbyAnim
            from: MyStyle.readyColor; to: MyStyle.standbyColor
            duration: 500
            running: false
        }
        Text {
            id: standbyText
            y: 0
            state: "standby"
            text: qsTr("STANDBY")
            anchors.left: parent.left
            anchors.leftMargin: 25
            anchors.verticalCenter: parent.verticalCenter
            z: 2
            font.pixelSize: 36
            color: "black"
        }
    }
    Rectangle {
        id: standbyAreaRound
        y: standbyArea.y
        width: 100
        height: 100
        color: standbyArea.color
        radius: 50
        anchors.right: standbyArea.left
        anchors.rightMargin: -50
        z: 0
    }
    //AIMING laser
    Rectangle {
        id: aimingAIntensityBorder
        x: 1015
        y: 709
        width: 120
        height: 60
        color: MyStyle.bgcolor
        z: 1
        border.width: 2
        border.color: MyStyle.borderColor
        Rectangle {
            x: 10
            y: 10
            id: pilot25
            width: 100
            height: 40
            color: "red"
            Text {
                id: aimBrightText
                anchors.fill: parent
                text: (mainWindow.aimIntensity/64*100).toFixed(0)+"%"
                color: "white"
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
            }
        }
    }

    Rectangle {
        id: aimAUp
        y: aimingAIntensityBorder.y
        width: 30
        height: 60
        color: MyStyle.borderColor
        anchors.left: aimingAIntensityBorder.right
        border.color: this.color
        border.width: 0
        Text {
            width: 30
            height: 60
            anchors.left: parent.left
            anchors.leftMargin: 10
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 2
            color: MyStyle.lightTextColor
            text: "+"
            z: 1
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 50
        }
        MouseArea {
            id: aimingBrightnessUpMouseArea
            anchors.top: parent.top
            anchors.right: aimIntensityUpRound.right
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            onClicked: {
                click()
                if(mainWindow.aimIntensity < 63) {
                    toggle(10)
                    aimingBrightnessUp.start()
                    this.enabled = false
                }
            }
            onPressAndHold: {
                click()
                aimBrightUpRep.start()
            }
            onPressed: if(mainWindow.aimIntensity < 63) {parent.color=MyStyle.pressedColor}
            onReleased: {
                if(mainWindow.aimIntensity < 63) {
                    aimBrightUpRep.stop()
                    if(aimBrightFlag) { toggle(10);aimBrightFlag=0 }
                }
                aimingBrightnessDownMouseArea.enabled = true
                parent.color=MyStyle.borderColor
            }
        }
        Rectangle {
            id: aimIntensityUpRound
            width: 60
            height: 60
            color: parent.color
            radius: 30
            border.color: parent.color
            border.width: 3
            anchors.left: parent.right
            anchors.leftMargin: -30
        }
    }
    Rectangle {
        id: aimADown
        y: aimingAIntensityBorder.y
        width: 30
        height: 60
        color: MyStyle.borderColor
        anchors.right: aimingAIntensityBorder.left
        border.color: this.color
        border.width: 0
        Text {
            width: 30
            height: 60
            z: 1
            anchors.left: parent.left
            anchors.leftMargin: -10
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 2
            color: MyStyle.lightTextColor
            text: "-"
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 50
        }
        MouseArea {
            id: aimingBrightnessDownMouseArea
            anchors.top: parent.top
            anchors.left: aimIntensityDownRound.left
            anchors.bottom: parent.bottom
            anchors.right: parent.right
            onClicked: {
                if(mainWindow.aimIntensity > 0) {
                    click()
                    toggle(11)
                    aimingBrightnessDown.start()
                    this.enabled = false
                }
            }
            onPressAndHold: {
                click()
                aimBrightDownRep.start()
            }
            onPressed: if(mainWindow.aimIntensity > 0) {parent.color=MyStyle.pressedColor}
            onReleased: {
                if(mainWindow.aimIntensity > 0) {
                    aimBrightDownRep.stop()
                    if(aimBrightFlag) { toggle(11);aimBrightFlag=0 }
                }
                aimingBrightnessDownMouseArea.enabled = true
                parent.color=MyStyle.borderColor
            }
        }
        Rectangle {
            id: aimIntensityDownRound
            width: 60
            height: 60
            color: parent.color
            radius: 30
            border.color: parent.color
            border.width: 3
            anchors.right: parent.left
            anchors.rightMargin: -30
        }
    }
    Rectangle {
        id: aimingBeam
        x: 975
        y: 579
        width: 200
        height: 50
        color: MyStyle.borderColor
        border.color: MyStyle.borderColor
        border.width: 2
        radius: 25
        Text {
            color: if(MyStyle.aimingButton) { MyStyle.lightTextColor } else { MyStyle.darkTextColor }
            text: qsTr("Aiming Beam")
            font.pointSize: 14
            anchors.fill: parent
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
        }
        MouseArea {
            id: aimingBeamMouseArea
            anchors.fill: parent
            onPressed: parent.color = MyStyle.pressedColor
            onClicked: {
                click()
                //Check if the button is on or off
                if(MyStyle.aimingButton) { //If the button is already on
                    MyStyle.aimingButton=false //turn it off
                    parent.color=MyStyle.bgcolor //visually represent this by changing the color of the button
                    if(MyStyle.aimOn) { //Also check if the aiming beam is currently on
                        MyStyle.aimOn = false //if so, turn it off
                        if(MyStyle.armMode) {toggle(8)} else {toggle(9)}
                    }
                } else { //If the button is not already on
                    MyStyle.aimingButton=true //Turn it on
                    parent.color=MyStyle.borderColor //visually represent this via button color
                    if(!MyStyle.aimOn) { //Check if aiming beam is currently off
                        if(MyStyle.armMode) {toggle(8)} else {toggle(9)} //if so, turn it on
                        MyStyle.aimOn = true
                    }
                }
            }

        }
    }
    Rectangle {
        id: aimMode
        x: 975
        y: 644
        width: 200
        height: 50
        radius: 25
        color: MyStyle.borderColor
        border.color: MyStyle.borderColor
        border.width: 2
        Text {
            id: aimModeText
            text: qsTr("CONT")
            color: MyStyle.lightTextColor
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 24
            anchors.fill: parent
        }
        MouseArea {
            id: aimModeMouseArea
            width: 200
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.bottom: parent.bottom
            onClicked: {
                swoosh()
                if(aimSlider.state == "blinking") {
                    aimSlider.state = "default"
                    aimModeText.text = "CONT"
                    blinking.stop() //Stop blinking timer
                    //Checks if aiming laser is supposed to be on
                    if(MyStyle.aimingButton) {
                        //If it is, check if the aiming beam is on
                        if(!MyStyle.aimOn) {
                            //if it is not, turn it on
                            if(MyStyle.armMode) {toggle(8)} else {toggle(9)}
                            MyStyle.aimOn = true
                        }
                    }
                } else {
                    aimSlider.state = "blinking"
                    aimModeText.text = "BLINKING"
                    blinking.start()
                }
            }
        }

    }

    Rectangle {
        id: aimSlider
        x: aimMode.x - 15
        y: aimMode.y -5
        width: 60
        height: 60
        color: MyStyle.sliderColor
        radius: 30
        z: 1
        states: [
            State {
                name: "blinking"
                PropertyChanges {
                    target: aimSlider
                    x: aimMode.x + 160
                }
            }
        ]
        transitions: [
            Transition {
                to: "blinking"
                reversible: true
                NumberAnimation {
                    properties: "x"
                    duration: 250
                    easing.type: Easing.InOutQuad }
            }
        ]
    }
    //this button/image triggers shuttle motion by toggling GPIO (J22) pin 3 and also GPIO (I2C) pin 12 (4)
    Rectangle {
        id: armFiberModeLight
        x: 940
        y: 250
        width: 270
        height: 220
        color: "#00000000"
        radius: 40
        border.width: 5
        border.color: MyStyle.armColor
        Rectangle {
            id: beamDeliveryFrame
            x: 10
            y: 10
            width: 250
            height: 200
            color: "#00000000"
            radius: 30
            border.width: 2
            border.color: MyStyle.borderColor
            Image {
                id: beamDeliveryDisplay
                anchors.fill: parent
                source: "Images/armIcon.png"
                fillMode: Image.PreserveAspectFit
            }
            Text {
                id: deliveryModeText
                color: MyStyle.darkTextColor
                text: "ARM"
                font.pointSize: 18
                horizontalAlignment: Text.AlignRight
                anchors.right: parent.right
                anchors.rightMargin: 20
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 3

            }
            MouseArea {
                id: armFiberSelectMouseArea
                anchors.fill: parent
                onClicked: {
                    this.enabled = false
                    click()
                    shuttleTO.start()
                    standbyAreaMouseArea.enabled = false
                    airMouseArea.enabled = false
                    gasMouseArea.enabled = false
                    toggle(3);toggle(12)
                }
            }
        }
    }
    //Slots that connect in main.cpp
    //Makes graphical changes to the laser mode section
    function modeChange(pgchange, serVal) {
        if (pgchange === true) {
            cwGraphic.color = "#FF9900"
            pulsedGraphic.strokeColor = MyStyle.darkTextColor
            cwBox.color = MyStyle.selectedBox
            spBox.color = "#00000000"
            if(serVal === 93) {
                cwPulseGraphic1.color = "#FF9900"
                cwPulseBox.color = MyStyle.selectedBox
                spPulseGraphic1.color = MyStyle.darkTextColor
                spPulseBox.color = "#00000000"
                sprPulseGraphic1.color = MyStyle.darkTextColor
                sprPulseBox.color = "#00000000"
            }
            if(serVal === 94) {
                cwPulseGraphic1.color = MyStyle.darkTextColor
                cwPulseBox.color = "#00000000"
                spPulseGraphic1.color = "#FF9900"
                spPulseBox.color = MyStyle.selectedBox
                sprPulseGraphic1.color = MyStyle.darkTextColor
                sprPulseBox.color = "#00000000"
            }
            if(serVal === 95) {
                cwPulseGraphic1.color = MyStyle.darkTextColor
                cwPulseBox.color = "#00000000"
                spPulseGraphic1.color = MyStyle.darkTextColor
                spPulseBox.color = "#00000000"
                sprPulseGraphic1.color = "#FF9900"
                sprPulseBox.color = MyStyle.selectedBox
            }
        } else {
            cwGraphic.color = MyStyle.darkTextColor
            pulsedGraphic.strokeColor = "#FF9900"
            cwBox.color = "#00000000"
            spBox.color = MyStyle.selectedBox
            if(serVal === 96){
                cwPulseGraphic1.color = "#FF9900"
                cwPulseBox.color = MyStyle.selectedBox
                spPulseGraphic1.color = MyStyle.darkTextColor
                spPulseBox.color = "#00000000"
                sprPulseGraphic1.color = MyStyle.darkTextColor
                sprPulseBox.color = "#00000000"
            }
            if(serVal === 97) {
                cwPulseGraphic1.color = MyStyle.darkTextColor
                cwPulseBox.color = "#00000000"
                spPulseGraphic1.color = "#FF9900"
                spPulseBox.color = MyStyle.selectedBox
                sprPulseGraphic1.color = MyStyle.darkTextColor
                sprPulseBox.color = "#00000000"
            }
            if(serVal === 98) {
                cwPulseGraphic1.color = MyStyle.darkTextColor
                cwPulseBox.color = "#00000000"
                spPulseGraphic1.color = MyStyle.darkTextColor
                spPulseBox.color = "#00000000"
                sprPulseGraphic1.color = "#FF9900"
                sprPulseBox.color = MyStyle.selectedBox
            }
        }
    }
    //Takes the passed value from the serialcontroller and updates the laser power display
    function powUpdate(pow_num) {
        powerDisplay.text = pow_num
        currentPower = pow_num //make it easier to track the currentPower in the system

    }
    //Takes the passed value from the serial controller to update the cycle time display
    function repeatUpdate(rep_num) {
        repeatDisplay.text = rep_num
    }
    //Takes the passed value from the serial controller to update the pulse width display
    function pulseUpdate(pul_num) {
        pulseDisplay.text = pul_num
    }
    //When the serial controller sends the current state of the system, update the machineState
    function standbyMode(rdy_bool) {
        if(rdy_bool) {
            MyStyle.machineState = "ready"
        } else {
            MyStyle.machineState = "standby"
        }
    }
    //Update the status bar when the serial controller sends an error code
    function errorMode(error_num) {
        switch(error_num) {
        case 40:
            log.append(getDate()+" | Interlock Error");
            if(!(MyStyle.errorValue&16)) { MyStyle.errorValue = MyStyle.errorValue + 16; console.log("Interlock Error") }
            break;
        case 41:
            statusText.text = "Coolant Error";
            log.append(getDate()+" | Coolant Error");
            break;
        case 42:
            log.append(getDate()+" | Shutter Error");
            if(!(MyStyle.errorValue&32)) { MyStyle.errorValue = MyStyle.errorValue + 32; console.log("Shutter Error") }
            break;
        case 43:
            log.append(getDate()+" | Attenuator Error");
            if(!(MyStyle.errorValue&128)) { MyStyle.errorValue = MyStyle.errorValue + 128; console.log("Attenuator Error") }
            break;
        case 44:
            log.append(getDate()+" | Footpedal Error");
            if(!(MyStyle.errorValue&64)) { MyStyle.errorValue = MyStyle.errorValue + 64; console.log("Footpedal Error") }
            break;
        case 45:
            statusText.text = "Power High Warning";
            log.append(getDate()+" | Power High Warning");
            break;
        case 46:
            statusText.text = "Power Low Warning";
            log.append(getDate()+" | Power Low Error");
            break;
        case 49:
            log.append(getDate()+" | Error Cleared");
            if(MyStyle.errorValue&128) { MyStyle.errorValue = MyStyle.errorValue - 128 }
            if(MyStyle.errorValue&64) { MyStyle.errorValue = MyStyle.errorValue - 64 }
            if(MyStyle.errorValue&32) { MyStyle.errorValue = MyStyle.errorValue - 32 }
            if(MyStyle.errorValue&16) { MyStyle.errorValue = MyStyle.errorValue - 16 }
            break;
        default:
            log.append(getDate()+" | Invalid Error Code");
            break;
        }
    }
    //Update the state of whether the system is lasing when the serial controller receives a lasing signal
    function lasing(lase) {
        if(lase) {
            MyStyle.lasing = true
            beep2()
            lasingSound.start()
        } else {
            MyStyle.lasing = false
            beep2()
            lasingSound.stop()
        }
    }
    //The following functions monitor the GPIO inputs via the gpiopin class and change when the value is changed.
    function fiberIn(val) { //If there is a fiber inserted or not
        if(val) {
            fiberStatus.active = true
            MyStyle.fiberStatus = true
        } else {
            fiberStatus.active = false
            MyStyle.fiberStatus = false
        }
    }
    function airIn(val) { //Whether or not the air pressure sensor is active
        if(val) {
            purgeTO.stop()
            airStatus.active = true
            standbyAreaMouseArea.enabled = true
        } else {
            airStatus.active = false
        }
    }
    function gasIn(val) { //Whether or not the external gas tank pressure sensor is active
        if(val) {
            purgeTO.stop()
            gasStatus.active = true
            standbyAreaMouseArea.enabled = true
        } else {
            gasStatus.active = false
        }
    }
    function laserIn(val) { //Whether or not the Laser is ready
        if(val) {
            laserStatus.active = true
            MyStyle.laserReady = true
        } else {
            laserStatus.active = false
            MyStyle.laserReady = false
        }
    }
    function interlockIn(val) { //If the interlock is OK
        if(val) {
            interlockStatus.active = true
            MyStyle.interlock = true
            if(MyStyle.errorValue&1){MyStyle.errorValue = MyStyle.errorValue - 1 }
            log.append(getDate()+" | HW Interlock Resolved (Key Turn)")
        } else {
            interlockStatus.active = false
            MyStyle.interlock = false
            if(!(MyStyle.errorValue&1)){MyStyle.errorValue = MyStyle.errorValue + 1 }
            log.append(getDate()+" | HW Interlock Error")
        }
    }
    function shuttleArm(val) { //The shuttle has hit the arm end position sensor
        if(val) {
            shuttleTO.stop()
            shuttleArmStatus.active = true
            MyStyle.shuttlePos = "arm"
            MyStyle.armMode = true
        } else {
            shuttleArmStatus.active = false
        }
    }
    function shuttleFiber(val) { //the shuttle has hit the fiber end position sensor
        if(val) {
            shuttleTO.stop()
            shuttleFiberStatus.active = true
            MyStyle.shuttlePos = "fiber"
            MyStyle.armMode = false
        } else {
            shuttleFiberStatus.active = false
        }
    }
    //This section monitors certain properties of the Global MyStyle variable that is imported and used in the QML file
    //Changes to a given value execute the coded actions.
    Connections {
        target: MyStyle
        function onArmModeChanged() {
            if(MyStyle.armMode) {
                beamDeliveryDisplay.source = "Images/armIcon.png"
                armFiberModeLight.border.color = MyStyle.armColor
                deliveryModeText.text = "ARM"
            } else {
                beamDeliveryDisplay.source = "Images/fiberIcon.png"
                armFiberModeLight.border.color = MyStyle.fiberColor
                deliveryModeText.text = "WAVEGUIDE"
                //Check that there is a fiber installed
                if(!MyStyle.fiberStatus) {
                    standbyAreaMouseArea.enabled = false
                }
            }
            if(MyStyle.aimOn) {toggle(8); toggle(9)}
        }
        function onLasingChanged() {
            if(MyStyle.lasing) { //if the system is lasing
                statusText.text = "LASING" //set the status bar to reflect that
                if(!MyStyle.pilotWhileOn) { //If the system is set to turn off the aiming laser while lasing...
                    MyStyle.aimHist = MyStyle.aimOn //save the current laser state to aimHist
                    blinking.stop() //stop blinking
                    if(MyStyle.aimOn) { //If it was on
                        if(MyStyle.armMode) {toggle(8)} else {toggle(9)} //turn it off
                        MyStyle.aimOn = false
                    }

                }
            } else { //otherwise revert
                statusBar.color = armFiberModeLight.border.color
                statusText.text = ""
                if(!MyStyle.pilotWhileOn) { //If the system is set to turn off the aiming laser while Lasing
                    if(MyStyle.aimHist) { //restore the inital state
                        if(MyStyle.armMode) {toggle(8)} else {toggle(9)} //and turn the laser on
                        MyStyle.aimOn = true
                        if(aimSlider.state=="blinking") { blinking.start() } //and start blinking if required
                    }
                    MyStyle.aimHist = false //reset Hysteresis
                }
            }
        }
        function onMachineStateChanged() {
            if(MyStyle.machineState === "ready") {
                rdyLtRdy.running = true
                readyAnim.running = true
                standbyText.text = "READY"
                powerUpButtonMouseArea.enabled = false
                powerDownButtonMouseArea.enabled = false
                pulseDownButtonMouseArea.enabled = false
                pulseUpButtonMouseArea.enabled = false
                repeatUpButtonMouseArea.enabled = false
                repeatDownButtonMouseArea.enabled = false
                armFiberSelectMouseArea.enabled = false
                modeButtonMouseArea.enabled = false
            } else {
                rdyLtStndby.running = true
                standbyAnim.running = true
                standbyText.text = "STANDBY"
                powerUpButtonMouseArea.enabled = true
                powerDownButtonMouseArea.enabled = true
                pulseDownButtonMouseArea.enabled = true
                pulseUpButtonMouseArea.enabled = true
                repeatUpButtonMouseArea.enabled = true
                repeatDownButtonMouseArea.enabled = true
                armFiberSelectMouseArea.enabled = true
                modeButtonMouseArea.enabled = true
            }
        }
        function onShuttlePosChanged() {
            if((MyStyle.shuttlePos ==="arm")|(MyStyle.shuttlePos ==="fiber")) {
                standbyAreaMouseArea.enabled = true
                airMouseArea.enabled = true
                gasMouseArea.enabled = true
                armFiberSelectMouseArea.enabled = true
                if((MyStyle.errorValue&2)) { statusText.text = "";statusBar.color = armFiberModeLight.border.color;log.append(getDate()+"Shuttle Error Cleared") }
            }
        }
        function onInterlockChanged() {
            if(MyStyle.interlock) {
                standbyAreaMouseArea.enabled = true
            } else {
                standbyAreaMouseArea.enabled = false
            }
        }
        function onAirStatusChanged() {
            if(MyStyle.airStatus) {
                airText.text = "Comp. Air On"
                air.color = MyStyle.borderColor
            } else {
                airText.text = "Comp. Air Off"
                air.color = MyStyle.bgcolor
            }
        }
        function onGasStatusChanged() {
            if(MyStyle.gasStatus) {
                gasText.text = "Gas On"
                gas.color = MyStyle.borderColor
            } else {
                gasText.text = "Gas Off"
                gas.color = MyStyle.bgcolor
            }
        }
        function onFiberStatusChanged() {
            if(!MyStyle.armMode) {
               if(!MyStyle.fiberStatus) {
                   standbyAreaMouseArea.enabled = false
               } else {
                   standbyAreaMouseArea.enabled = true
               }
            }
        }
        function onErrorValueChanged() {
            console.log("Error Value Changed - New Value: "+ MyStyle.errorValue)
            if(MyStyle.errorValue > 0) {
                statusBar.color = "red"
                standbyAreaMouseArea.enabled = false
                errorMessages.start()
            } else {
                statusBar.color = armFiberModeLight.border.color
                statusText.text = ""
                standbyAreaMouseArea.enabled = true
                errorMessages.stop();counter=0
            }
        }
    }
}
